import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:flutter_facebook_auth/flutter_facebook_auth.dart';
import 'package:login_firebase/common/ModErrorCodes.dart';
import 'package:login_firebase/common/ModLogger.dart';
import 'package:login_firebase/models/ThirdPartySessionModel.dart';
import 'package:login_firebase/presentation/factories/LoginIconButtonFactory.dart';
import 'package:login_firebase/presentation/factories/LoginWidgetFactory.dart';
import 'package:login_firebase/presentation/resources/LoginColors.dart';

const TAG = "FacebookSigninWidget";


class FacebookSigninWidget extends StatefulWidget {
  final Function(ThirdPartySessionModel) onSessionSuccess;
  final Function(int) onSessionError;
  final Function() onReset;
  final Function(String)? onDisplay;
  final bool available;
  final String btnText;
  final String fbId;

  const FacebookSigninWidget({
    required this.onSessionSuccess,
    required this.onSessionError,
    required this.onReset,
    required this.btnText,
    required this.fbId,
    this.onDisplay,
    this.available = true,
    super.key,
  });

  @override
  State<StatefulWidget> createState() => _FacebookSigninWidgetState();
}

class _FacebookSigninWidgetState extends State<FacebookSigninWidget> {
  void _login() async {
    try {
      LoginResult loginResult = await FacebookAuth.instance.login();
      AccessToken? accessToken = loginResult.accessToken;
      // RES
      if (accessToken != null) {
        final fbCred = FacebookAuthProvider.credential(accessToken.token);
        final user = await FirebaseAuth.instance.signInWithCredential(fbCred);
        if (user.user != null) {
          final token = await user.user!.getIdToken();

          widget.onSessionSuccess.call(ThirdPartySessionModel(
                mail: user.user!.email,
                token: token!,
                display: user.user!.displayName,
                pic: user.user!.photoURL,
                mailValidationDone: true,
              ));
          return;
        }
      }

      // NO RES
      widget
          .onSessionSuccess
          .call(ThirdPartySessionModel.requiresMailValidation());
    } on Exception catch (e) {
      ModLogger.e(tag: TAG, msg: "_processResult() ${e.toString()}", error: e);

      widget.onSessionError.call(ModErrorCodes.FACEBOOK_SIGNIN);
    }

    setState(() {}); //XXX: launches rebuild and postFrame on External
  }

  @override
  Widget build(BuildContext context) {
    return _wrap(_buildButton(context));
  }

  Widget _wrap(Widget child) => Container(
      margin: const EdgeInsets.symmetric(horizontal: 8.0),
      padding: const EdgeInsets.all(4.0),
      child: Center(child: child));

  Widget _buildButton(BuildContext context) =>
      LoginWidgetFactory.buildSigninBtn(
          context: context,
          containerColor: LoginColors.blueColor,
          borderColor: LoginColors.blueColor,
          icon:
              LoginIconButtonFactory.getFacebookIcon(height: 25.0, width: 25.0),
          text: widget.btnText,
          textColor: LoginColors.whiteColor,
          onClick: widget.available
              ?

              // AVAIL
              () {
                  widget.onReset.call();
                  _login();
                }
              :
              // NOT AVAIL
              () {
                  widget
                      .onSessionError
                      .call(ModErrorCodes.SIGNIN_NOT_AVAILABLE);
                });
}
